# -*- coding: utf-8 -*

from paths import ROOT_PATH  # isort:skip
from videoanalyst.config.config import cfg
from videoanalyst.config.config import specify_task
from videoanalyst.model import builder as model_builder
from videoanalyst.pipeline import builder as pipeline_builder
from videoanalyst.utils import complete_path_wt_root_in_cfg
from videoanalyst.pipeline.utils.bbox import xywh2xyxy, xyxy2xywh

import argparse
from loguru import logger

import cv2
import numpy as np
import time
import torch
import vot
import os
import sys
import os


from easydict import EasyDict
import json

# need to config
tracker_path='path_to_tracker'
def add_path(path):
    if path not in sys.path:
        sys.path.insert(0,path)
this_dir=os.path.join(tracker_path)
add_path(this_dir)
this_dir=os.path.join(tracker_path,'Seger')
add_path(this_dir)
import Box2Seg
def _rect_from_mask(self, mask):
    '''
    create an axis-aligned rectangle from a given binary mask
    mask in created as a minimal rectangle containing all non-zero pixels
    '''
    x_ = np.sum(mask, axis=0)
    y_ = np.sum(mask, axis=1)
    x0 = np.min(np.nonzero(x_))
    x1 = np.max(np.nonzero(x_))
    y0 = np.min(np.nonzero(y_))
    y1 = np.max(np.nonzero(y_))
    return [x0, y0, x1 - x0 + 1, y1 - y0 + 1]

def make_full_size(x, output_sz):
    '''
    zero-pad input x (right and down) to match output_sz
    x: numpy array e.g., binary mask
    output_sz: size of the output [width, height]
    '''
    if x.shape[0] == output_sz[1] and x.shape[1] == output_sz[0]:
        return x
    pad_x = output_sz[0] - x.shape[1]
    if pad_x < 0:
        x = x[:, :x.shape[1] + pad_x]
        # padding has to be set to zero, otherwise pad function fails
        pad_x = 0
    pad_y = output_sz[1] - x.shape[0]
    if pad_y < 0:
        x = x[:x.shape[0] + pad_y, :]
        # padding has to be set to zero, otherwise pad function fails
        pad_y = 0
    return np.pad(x, ((0, pad_y), (0, pad_x)), 'constant', constant_values=0)


def _rect_from_mask(mask):
    '''
    create an axis-aligned rectangle from a given binary mask
    mask in created as a minimal rectangle containing all non-zero pixels
    '''
    x_ = np.sum(mask, axis=0)
    y_ = np.sum(mask, axis=1)
    x0 = np.min(np.nonzero(x_))
    x1 = np.max(np.nonzero(x_))
    y0 = np.min(np.nonzero(y_))
    y1 = np.max(np.nonzero(y_))
    return [x0, y0, x1 - x0 + 1, y1 - y0 + 1]

class SiamEM(object):
    def __init__(self, image, mask):
        self.use_seg = True
        root_cfg = cfg
        root_cfg.merge_from_file( os.path.join(tracker_path,"experiments/siamfcpp/test/vot/siamfcpp_alexnet.yaml"))
        root_cfg = complete_path_wt_root_in_cfg(root_cfg, ROOT_PATH)
        root_cfg = root_cfg.test
        task, task_cfg = specify_task(root_cfg)
        task_cfg.freeze()

        if self.use_seg:
            config_deeplab_path =os.path.join(tracker_path, 'Seger/config.json')
            seg_args = EasyDict(json.load(open(config_deeplab_path))['inference'])
            seg_args.model=os.path.join(tracker_path,'Seger/models/deeplab-resnet.pth.tar')
            self.seger = Box2Seg.Seger(seg_args)
        region = _rect_from_mask(mask)
        # build model
        self.model = model_builder.build(task, task_cfg.model)
        # build pipeline
        self.pipeline = pipeline_builder.build(task, task_cfg.pipeline, self.model)
        dev = torch.device("cuda")
        self.pipeline.set_device(dev)
        # init frame
        self.pipeline.init(image, region)
    def track(self, image):
        frame = image.copy()
        rect_pred, ps = self.pipeline.update(image)
        rect_for_seg = xywh2xyxy(rect_pred.copy(),image.shape)

        rect_for_seg = list(map(int, rect_for_seg))
        if self.use_seg:
            frame = image.copy()
            msk = self.seger.inference(frame[rect_for_seg[1]:rect_for_seg[3], rect_for_seg[0]:rect_for_seg[2]])
        # return vot.Rectangle(rect_pred[0], rect_pred[1], rect_pred[2], rect_pred[3]), ps

        output_sz = frame.shape
        mask = np.zeros((output_sz[0], output_sz[1]), dtype=np.uint8)
        mask[rect_for_seg[1]:rect_for_seg[3], rect_for_seg[0]:rect_for_seg[2]] = msk

        return mask, ps

handle = vot.VOT("mask")
selection = handle.region()
imagefile = handle.frame()
image = cv2.imread(imagefile)
mask = make_full_size(selection, (image.shape[1], image.shape[0]))



if not imagefile:
    sys.exit(0)

tracker = SiamEM(image, mask)

while True:
    imagefile = handle.frame()
    if not imagefile:
        break
    image = cv2.imread(imagefile)
    m, confidence = tracker.track(image)
    # f = open('/home/lyz/data/vot_test/boxs.txt', 'a')
    # print(111, file=f)
    # f.close()
    handle.report(m, confidence)

