# -*- coding: utf-8 -*

from paths import ROOT_PATH  # isort:skip
from videoanalyst.config.config import cfg
from videoanalyst.config.config import specify_task
from videoanalyst.model import builder as model_builder
from videoanalyst.pipeline import builder as pipeline_builder
from videoanalyst.utils import complete_path_wt_root_in_cfg
from videoanalyst.pipeline.utils.bbox import xywh2xyxy, xyxy2xywh
from torchvision.transforms import ToPILImage
import json

from easydict import EasyDict
import argparse
from loguru import logger
import glob
import cv2
import numpy as np
import time
import torch
import os
import sys
import Box2Seg
font_size = 0.5
font_width = 1


def make_parser():
    parser = argparse.ArgumentParser(
        description="press s to select the target box,\n \
                        then press enter or space to confirm it or press c to cancel it,\n \
                        press c to stop track and press q to exit program")
    parser.add_argument(
        "-cfg",
        "--config",
        default="experiments/siamfcpp/test/got10k/siamfcpp_alexnet-got.yaml",
        type=str,
        help='experiment configuration')
    parser.add_argument("-d",
                        "--device",
                        default="cpu",
                        type=str,
                        help="torch.device, cuda or cpu")
    parser.add_argument("-v",
                        "--video",
                        type=str,
                        default="webcam",
                        help="path to input video file, default is webcam")
    parser.add_argument("-o",
                        "--output",
                        type=str,
                        default="",
                        help="path to dump the track video")
    return parser


def main(args):
    use_seg=True
    root_cfg = cfg
    root_cfg.merge_from_file("/home/lyz/data/video_analyst-master/experiments/siamfcpp/test/vot/siamfcpp_alexnet.yaml")
    logger.info("Load experiment configuration at: %s" % args.config)

    # resolve config
    root_cfg = complete_path_wt_root_in_cfg(root_cfg, ROOT_PATH)
    root_cfg = root_cfg.test
    task, task_cfg = specify_task(root_cfg)
    task_cfg.freeze()
    window_name = task_cfg.exp_name
    # build model
    model = model_builder.build(task, task_cfg.model)
    # build pipeline
    pipeline = pipeline_builder.build(task, task_cfg.pipeline, model)
    dev = torch.device(args.device)
    pipeline.set_device(dev)
    init_box = None
    template = None
    vw = None
    # args.video='/home/lyz/data/test_video/drone1'
    args.video= '/home/lyz/data/vot_test/sequences/fernando/color'
    filenames=sorted(glob.glob(os.path.join(args.video,"*jpg")),key=lambda x:int(os.path.basename(x).split('.')[0]))
    frames=[cv2.imread(filename) for filename in filenames]
    # if args.video == "webcam":
    #     logger.info("[INFO] starting video stream...")
    #     vs = cv2.VideoCapture(0)
    #     vs.set(cv2.CAP_PROP_FOURCC, cv2.VideoWriter_fourcc('M', 'J', 'P', 'G'))
    # else:
    #     vs = cv2.VideoCapture(args.video)
    if use_seg:
        config_deeplab_path = '/home/lyz/data/SiamEM/Seger/config.json'
        seg_args = EasyDict(json.load(open(config_deeplab_path))['inference'])
        seger = Box2Seg.Seger(seg_args)
    if args.output:
        fourcc = cv2.VideoWriter_fourcc(*'MJPG')
        width, height = frames[0].get(3), frames[0].get(4)
        vw = cv2.VideoWriter(args.output, fourcc, 25, (int(width), int(height)))
    for index,frame in enumerate(frames):

        if index==0:
            # select the bounding box of the object we want to track (make
            # sure you press ENTER or SPACE after selecting the ROI)
            box = cv2.selectROI(window_name,
                                frame,
                                fromCenter=False,
                                showCrosshair=True)
            if box[2] > 0 and box[3] > 0:
                init_box = box
                template = cv2.resize(
                    frame[box[1]:box[1] + box[3], box[0]:box[0] + box[2]],
                    (128, 128))
                pipeline.init(frame, init_box)

        else :
            time_a = time.time()
            rect_pred = pipeline.update(frame)
            show_frame = frame.copy()
            time_cost = time.time() - time_a
            bbox_pred = xywh2xyxy(rect_pred[0],frame.shape)
            bbox_pred = tuple(map(int, bbox_pred))
            if use_seg:

                seger.inference(frame[bbox_pred[1]:bbox_pred[3],bbox_pred[0]:bbox_pred[2]])

            cv2.putText(show_frame,
                        "track cost: {:.4f} s".format(time_cost), (128, 20),
                        cv2.FONT_HERSHEY_COMPLEX, font_size, (0, 0, 255),
                        font_width)
            cv2.rectangle(show_frame, bbox_pred[:2], bbox_pred[2:],
                          (0, 255, 0))
            if template is not None:
                show_frame[:128, :128] = template


            cv2.imshow(window_name, show_frame)
        if vw is not None:
            vw.write(show_frame)
        key = cv2.waitKey(30) & 0xFF
        if key == ord("q"):
            break
        # if the 's' key is selected, we are going to "select" a bounding
        # box to track

        elif key == ord("c"):
            init_box = None
            template = None

    if vw is not None:
        vw.release()
    cv2.destroyAllWindows()


if __name__ == "__main__":
    parser = make_parser()
    args = parser.parse_args()
    main(args)
